package com.sagacloud.extend.dao_data;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.sagacloud.util.compute_engine.core.AnnotationDescription;
import com.sagacloud.util.compute_engine.core.IDao_data;
import com.sagacloud.util.compute_engine.util.ConstantCE;
import com.sagacloud.util.compute_engine.util.DoubleTimeObject;
import com.sagacloud.util.compute_engine.util.QueryUtil;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@AnnotationDescription(value = "dao_data")
public class ImplDao_tenantfee implements IDao_data {

    String database_name;
    String table_name;
    String dao_type = "tenantfee";

    @Override
    public void Initialize(String database_name) {

        this.database_name = database_name;
        this.table_name = "data_" + dao_type;
    }

    @Override
    public String GetName() {
        return this.dao_type;
    }

    @Override
    public List<DoubleTimeObject> GetDataListGteLte(JSONObject dao_key, String time_period, Date time_from, Date time_to, boolean GteLte)
            throws Exception {
        List<DoubleTimeObject> result = null;

        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
        String building = (String) dao_key.get("building");
        String tenant = (String) dao_key.get("tenant");
        String energy_type = (String) dao_key.get("energy_type");
        String meter_sign = (String) dao_key.get("meter_sign");
        Long funcid = (Long) dao_key.get("funcid");

        JSONObject Criteria = new JSONObject();
        Criteria.put("building", building);
        Criteria.put("tenant", tenant);
        Criteria.put("energy_type", energy_type);
        Criteria.put("meter_sign", meter_sign);
        Criteria.put("funcid", funcid);
        if (ConstantCE.time_period_seperate) {
            //
        } else {
            Criteria.put("time_period", time_period);
        }
        JSONObject range = new JSONObject();
        range.put("$gte", sdf.format(time_from));
        if (time_to != null) {
            if (GteLte) {
                range.put("$lte", sdf.format(time_to));
            } else {
                range.put("$lt", sdf.format(time_to));
            }
        }
        Criteria.put("data_time", range);
        String table_name_query;
        if (ConstantCE.time_period_seperate) {
            table_name_query = table_name + "_" + time_period;
        } else {
            table_name_query = table_name;
        }
        JSONArray resultContent = QueryUtil.select(database_name, table_name_query, Criteria);
        List<DoubleTimeObject> dataList = new ArrayList<DoubleTimeObject>();
        for (int ii = 0; ii < resultContent.size(); ii++) {
            JSONObject computelog = (JSONObject) resultContent.get(ii);
            DoubleTimeObject data = new DoubleTimeObject();
            data.data_time = sdf.parse((String) computelog.get("data_time"));
            data.data_value = (Double) computelog.get("data_value");
            dataList.add(data);
        }
        result = dataList;

        return result;
    }

    @Override
    public void InsertDataListGteLte(JSONObject dao_key, String time_period, List<DoubleTimeObject> outputDataList) throws Exception {
        if (ConstantCE.time_period_seperate_save_two || !ConstantCE.time_period_seperate) {
            this.InsertDataListGteLte(false, dao_key, time_period, outputDataList);
        }
        if (ConstantCE.time_period_seperate_save_two || ConstantCE.time_period_seperate) {
            this.InsertDataListGteLte(true, dao_key, time_period, outputDataList);
        }
    }

    @Override
    public void DeleteDataListGteLte(JSONObject dao_key, String time_period, Date time_from, Date time_to) throws Exception {
        if (ConstantCE.time_period_seperate_save_two || !ConstantCE.time_period_seperate) {
            this.DeleteDataListGteLte(false, dao_key, time_period, time_from, time_to);
        }
        if (ConstantCE.time_period_seperate_save_two || ConstantCE.time_period_seperate) {
            this.DeleteDataListGteLte(true, dao_key, time_period, time_from, time_to);
        }
    }

    private void DeleteDataListGteLte(boolean time_period_seperate, JSONObject dao_key, String time_period, Date time_from, Date time_to)
            throws Exception {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");

        String building = (String) dao_key.get("building");
        String tenant = (String) dao_key.get("tenant");
        String energy_type = (String) dao_key.get("energy_type");
        String meter_sign = (String) dao_key.get("meter_sign");
        Long funcid = (Long) dao_key.get("funcid");

        JSONObject Criteria = new JSONObject();
        Criteria.put("building", building);
        Criteria.put("tenant", tenant);
        Criteria.put("energy_type", energy_type);
        Criteria.put("meter_sign", meter_sign);
        Criteria.put("funcid", funcid);
        if (time_period_seperate) {
            //
        } else {
            Criteria.put("time_period", time_period);
        }
        JSONObject range = new JSONObject();
        range.put("$gte", sdf.format(time_from));
        range.put("$lte", sdf.format(time_to));
        Criteria.put("data_time", range);
        String table_name_query;
        if (time_period_seperate) {
            table_name_query = table_name + "_" + time_period;
        } else {
            table_name_query = table_name;
        }
        QueryUtil.delete(database_name, table_name_query, Criteria);
    }

    private void InsertDataListGteLte(boolean time_period_seperate, JSONObject dao_key, String time_period, List<DoubleTimeObject> outputDataList)
            throws Exception {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");

        String building = (String) dao_key.get("building");
        String tenant = (String) dao_key.get("tenant");
        String energy_type = (String) dao_key.get("energy_type");
        String meter_sign = (String) dao_key.get("meter_sign");
        Long funcid = (Long) dao_key.get("funcid");

        JSONArray InsertObjects = new JSONArray();
        for (DoubleTimeObject data : outputDataList) {
            JSONObject InsertObject = new JSONObject();
            InsertObject.put("building", building);
            InsertObject.put("tenant", tenant);
            InsertObject.put("energy_type", energy_type);
            InsertObject.put("meter_sign", meter_sign);
            InsertObject.put("funcid", funcid);
            if (time_period_seperate) {
                //
            } else {
                InsertObject.put("time_period", time_period);
            }
            InsertObject.put("data_time", sdf.format(data.data_time));
            InsertObject.put("data_value", data.data_value);
            InsertObjects.add(InsertObject);
        }
        String table_name_query;
        if (time_period_seperate) {
            table_name_query = table_name + "_" + time_period;
        } else {
            table_name_query = table_name;
        }
        QueryUtil.batch_insert(database_name, table_name_query, InsertObjects);
    }
}
