package com.sagacloud.util.compute_engine.fjd;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.sagacloud.repository.HBaseSaveManagerRepository;
import com.sagacloud.util.compute_engine.util.ConstantCE;
import com.sagacloud.util.compute_engine.util.DateAccelerator;
import com.sagacloud.util.compute_engine.util.QueryUtil;
import com.zillion.database.agent.ZillionAgent;
import com.zillion.util.zk.ZKWrapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.zookeeper.CreateMode;
import org.apache.zookeeper.data.Stat;

import java.nio.charset.StandardCharsets;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Slf4j
public class ThreadDelay extends Thread {
    static List<JSONObject> jobList = new ArrayList<JSONObject>();
    ZillionAgent agent;
    String database_name;
    List<ThreadDelayItem> threadList;
    List<ThreadDelayStat> threadStatList;
    String path_delay = ConstantCE.zk_root + "/" + "delay";
    String path_delay_lock = ConstantCE.zk_root + "/" + "delay_lock";
    SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
    private volatile boolean stop = false;

    public ThreadDelay(String database_name) {

        this.database_name = database_name;
    }

    public static synchronized JSONObject Pop() {
        JSONObject result = null;
        if (jobList.size() > 0) {
            result = jobList.remove(0);
        }
        return result;
    }

    public static synchronized void Push(JSONObject job) {
        jobList.add(job);
    }

    public void requestStop() {
        stop = true;
    }

    @Override
    public void run() {
        // DateAccelerator.Set(600);
        ZKWrapper ZK = HBaseSaveManagerRepository.randomZillionAgent().container.ZK;
        String[] roots = {path_delay, path_delay_lock};
        for (String root : roots) {
            try {
                Stat stat = ZK.exists(root, false);
                if (stat == null) {
                    try {
                        ZK.create(root, "true".getBytes(StandardCharsets.UTF_8), CreateMode.PERSISTENT);
                    } catch (Exception e) {
                       log.error(e.getMessage(),e);
                    }
                }
            } catch (Exception e2) {
                log.error(e2.getMessage(),e2);
            }
        }

        threadList = new ArrayList<ThreadDelayItem>();
        for (int i = 0; i < ConstantCE.original_delay_thread_count; i++) {
            ThreadDelayItem thread = new ThreadDelayItem(ConstantCE.database_name);
            thread.start();
            threadList.add(thread);
        }
        int stat_count = (int) Math.sqrt(ConstantCE.original_delay_thread_count);
        stat_count = stat_count < 1 ? 1 : stat_count;
        threadStatList = new ArrayList<ThreadDelayStat>();
        for (int i = 0; i < stat_count; i++) {
            ThreadDelayStat thread = new ThreadDelayStat(ConstantCE.database_name);
            thread.start();
            threadStatList.add(thread);
        }

        while (!stop) {
            Date curr_proc_time = new Date(DateAccelerator.Get().getTime() / (1000L * 60 * ConstantCE.original_delay_period_minute)
                    * (1000L * 60 * ConstantCE.original_delay_period_minute) - (1000L * 60 * ConstantCE.original_delay_period_minute));
            try {
                List<String> Children = ZK.getChildren(path_delay, false);
                JSONObject Criteria = new JSONObject();
                JSONArray Content = QueryUtil.select(database_name, "fjd_0_buildingcomputetime", Criteria);
                for (int i = 0; i < Content.size(); i++) {
                    JSONObject item = (JSONObject) Content.get(i);
                    String building = (String) item.get("building");
                    boolean exist = false;
                    for (String child : Children) {
                        if (child.equals(building)) {
                            exist = true;
                            break;
                        }
                    }
                    if (!exist) {
                        try {
                            ZK.create(path_delay + "/" + building, sdf.format(curr_proc_time).getBytes(), CreateMode.PERSISTENT);
                        } catch (Exception e) {
                           log.error(e.getMessage(),e);
                        }
                    }
                }
            } catch (Exception e) {
                log.error(e.getMessage(), e);
            }

            try {
                Thread.sleep(1000L * 60);
            } catch (InterruptedException e1) {
                log.error(e1.getMessage(),e1);
            }
        }

        for (int i = 0; i < threadList.size(); i++) {
            ThreadDelayItem thread = threadList.get(i);
            thread.requestStop();
        }
        for (int i = 0; i < threadStatList.size(); i++) {
            ThreadDelayStat thread = threadStatList.get(i);
            thread.requestStop();
        }
        while (true) {
            try {
                Thread.sleep(1L);
            } catch (InterruptedException e) {
                log.error(e.getMessage(),e);
            }

            boolean all_finish1 = true;
            for (ThreadDelayItem thread : threadList) {
                if (thread.isAlive()) {
                    all_finish1 = false;
                    break;
                }
            }
            boolean all_finish2 = true;
            for (ThreadDelayStat thread : threadStatList) {
                if (thread.isAlive()) {
                    all_finish2 = false;
                    break;
                }
            }
            if (all_finish1 && all_finish2) {
                break;
            }
        }
    }
}