package test;

import com.alibaba.fastjson.JSONObject;
import com.sagacloud.extend.dao_data.ImplDao_fjd_0;
import com.sagacloud.extend.dao_data.ImplDao_fjd_near;
import com.sagacloud.extend.dao_data.ImplDao_ori;
import com.sagacloud.extend.dao_data.ImplDao_servicedata;
import com.sagacloud.util.compute_engine.util.DateUtil;
import com.sagacloud.util.compute_engine.util.DoubleTimeObject;
import com.sagacloud.util.compute_engine.util.DoubleTimeObject_fjd_near;
import com.zillion.database.agent.ZillionAgent;
import lombok.extern.slf4j.Slf4j;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Slf4j
public class Check_servicedata_Thread extends Thread {
    SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");

    ZillionAgent agent;
    String database_name;

    public Check_servicedata_Thread(String database_name) {

        this.database_name = database_name;
    }

    @Override
    public void run() {
        ImplDao_ori dao_ori = new ImplDao_ori();
        dao_ori.Initialize(database_name);
        ImplDao_fjd_0 dao_fjd_0 = new ImplDao_fjd_0();
        dao_fjd_0.Initialize(database_name);

        ImplDao_fjd_near dao_fjd_near = new ImplDao_fjd_near();
        dao_fjd_near.Initialize(database_name);
        ImplDao_servicedata dao_servicedata = new ImplDao_servicedata();
        dao_servicedata.Initialize(database_name);

        while (true) {
            JSONObject item = Check_servicedata.Pop();
            if (item == null) {
                break;
            }

            try {
                this.meterdata(item, dao_ori, dao_fjd_0);
                this.servicedata(item, dao_fjd_near, dao_servicedata);
            } catch (Exception e) {
                log.error(e.getMessage(), e);
            }
        }
    }

    private void meterdata(JSONObject item, ImplDao_ori dao_ori, ImplDao_fjd_0 dao_fjd_0) throws Exception {
        String time_period = "15min";
        long interval = DateUtil.Instance().get_about(time_period);
        int seek_period_count = 2;
        Date timefrom = sdf.parse(Check_servicedata.time_from);
        Date timeto = sdf.parse(Check_servicedata.time_to);

        String building = (String) item.get("building");
        String meter = (String) item.get("meter");
        Long funcid = (Long) item.get("funcid");
        List<DoubleTimeObject> dataList_ori = dao_ori.GetDataListGteLte(item, time_period,
                DateUtil.Instance().GetDate_offset(timefrom, time_period, -seek_period_count),
                DateUtil.Instance().GetDate_offset(timeto, time_period, seek_period_count), true);
        List<DoubleTimeObject> dataList_fjd_0 = dao_fjd_0.GetDataListGteLte(item, time_period, timefrom, timeto, false);

        // 准备数据
        List<DoubleTimeObject> inputDataList = dataList_ori;

        // 计算
        int index_tmp = 0;
        List<DoubleTimeObject> outputDataList = new ArrayList<DoubleTimeObject>();
        for (Date tmpDate = timefrom; tmpDate.compareTo(timeto) < 0; tmpDate = DateUtil.Instance().GetDate_offset(tmpDate, time_period, 1)) {
            DoubleTimeObject data_best = null;
            DoubleTimeObject data_before = null;
            DoubleTimeObject data_after = null;
            for (int index_data = index_tmp; index_data < inputDataList.size(); index_data++) {
                DoubleTimeObject data = inputDataList.get(index_data);
                if (data.data_time.getTime() == tmpDate.getTime()) {
                    index_tmp = index_data;
                    data_best = data;
                    break;
                } else if (data.data_time.getTime() < tmpDate.getTime()) {
                    index_tmp = index_data;
                    if (tmpDate.getTime() - data.data_time.getTime() <= interval * seek_period_count) {
                        data_before = data;
                    }
                } else {
                    if (data.data_time.getTime() - tmpDate.getTime() <= interval * seek_period_count) {
                        data_after = data;
                    }
                    break;
                }
            }

            if (data_best != null) {
                DoubleTimeObject outputData = new DoubleTimeObject();
                outputData.data_time = tmpDate;
                outputData.data_value = data_best.data_value;
                outputDataList.add(outputData);
            } else if (data_before != null && data_after != null) {
                double data_value = data_before.data_value + (1.0 * (tmpDate.getTime() - data_before.data_time.getTime())
                        / (data_after.data_time.getTime() - data_before.data_time.getTime())) * (data_after.data_value - data_before.data_value);
                DoubleTimeObject outputData = new DoubleTimeObject();
                outputData.data_time = tmpDate;
                outputData.data_value = data_value;
                outputDataList.add(outputData);
            }
        }

        Date error_from = null;
        Date error_to = null;
        int index_des = 0;
        for (int index_out = 0; index_out < outputDataList.size(); index_out++) {
            DoubleTimeObject outputData = outputDataList.get(index_out);
            DoubleTimeObject fjd_0 = null;
            for (int i = index_des; i < dataList_fjd_0.size(); i++) {
                index_des = i;
                DoubleTimeObject data = dataList_fjd_0.get(i);
                if (data.data_time.getTime() > outputData.data_time.getTime()) {
                    break;
                } else if (data.data_time.getTime() == outputData.data_time.getTime()) {
                    fjd_0 = data;
                    break;
                }
            }
            if (fjd_0 == null) {
                if (error_from == null) {
                    error_from = outputData.data_time;
                }
                error_to = outputData.data_time;
            }
        }
        if (error_from != null) {
            log.warn(
                    building + "\t" + meter + "\t" + funcid + "\t" + "ori!=meter" + "\t" + sdf.format(error_from) + "\t" + sdf.format(error_to));
        }
    }

    private void servicedata(JSONObject item, ImplDao_fjd_near dao_fjd_near, ImplDao_servicedata dao_servicedata) throws Exception {
        String building = (String) item.get("building");
        String meter = (String) item.get("meter");
        Long funcid = (Long) item.get("funcid");
        List<DoubleTimeObject_fjd_near> dataList_fjd = dao_fjd_near.GetDataListGteLte_fjd_near(item, "15min", sdf.parse(Check_servicedata.time_from),
                sdf.parse(Check_servicedata.time_to), true);
        List<DoubleTimeObject> dataList_servicedata_15min = dao_servicedata.GetDataListGteLte(item, "15min", sdf.parse(Check_servicedata.time_from),
                sdf.parse(Check_servicedata.time_to), false);
        Date error_from = null;
        Date error_to = null;
        int index_des = 0;
        for (int index_fjd = 0; index_fjd < dataList_fjd.size() - 1; index_fjd++) {
            DoubleTimeObject_fjd_near data = dataList_fjd.get(index_fjd);
            DoubleTimeObject_fjd_near data_next = dataList_fjd.get(index_fjd + 1);
            if (data_next.data_time.getTime() - data.data_time.getTime() == 1000L * 60 * 15) {
                DoubleTimeObject data_servicedata = null;
                for (int i = index_des; i < dataList_servicedata_15min.size(); i++) {
                    index_des = i;
                    DoubleTimeObject servicedata = dataList_servicedata_15min.get(i);
                    if (servicedata.data_time.getTime() > data.data_time.getTime()) {
                        break;
                    } else if (servicedata.data_time.getTime() == data.data_time.getTime()) {
                        data_servicedata = servicedata;
                        break;
                    }
                }
                if (data_servicedata == null) {
                    if (error_from == null) {
                        error_from = data.data_time;
                    }
                    error_to = data.data_time;
                }
            }
        }
        if (error_from != null) {
            log.warn(
                    building + "\t" + meter + "\t" + funcid + "\t" + "meter!=service" + "\t" + sdf.format(error_from) + "\t" + sdf.format(error_to));
        }

        boolean check_stat = true;
        if (check_stat) {
            List<DoubleTimeObject> dataList_servicedata_1h = dao_servicedata.GetDataListGteLte(item, "1h", sdf.parse(Check_servicedata.time_from),
                    sdf.parse(Check_servicedata.time_to), false);
            List<DoubleTimeObject> dataList_servicedata_1d = dao_servicedata.GetDataListGteLte(item, "1d", sdf.parse(Check_servicedata.time_from),
                    sdf.parse(Check_servicedata.time_to), false);
            if (dataList_servicedata_15min.size() > 0) {
                DoubleTimeObject data_first = dataList_servicedata_15min.get(0);
                DoubleTimeObject data_final = dataList_servicedata_15min.get(dataList_servicedata_15min.size() - 1);
                Date time_from_1h = DateUtil.Instance().get_floor(data_first.data_time, "1h");
                Date time_to_1h = DateUtil.Instance().get_floor(data_final.data_time, "1h");
                Date time_from_1d = DateUtil.Instance().get_floor(data_first.data_time, "1d");
                Date time_to_1d = DateUtil.Instance().get_floor(data_final.data_time, "1d");

                this.Match(building, meter, funcid, "1h", time_from_1h, time_to_1h, dataList_servicedata_15min, dataList_servicedata_1h);
                this.Match(building, meter, funcid, "1d", time_from_1d, time_to_1d, dataList_servicedata_15min, dataList_servicedata_1d);
            }
        }
    }

    private void Match(String building, String meter, Long funcid, String time_period, Date time_from, Date time_to,
                       List<DoubleTimeObject> dataList_servicedata_src, List<DoubleTimeObject> dataList_servicedata_des) {
        long interval = DateUtil.Instance().get_about(time_period);
        Date error_from = null;
        Date error_to = null;
        int index_src = 0;
        int index_des = 0;
        Date time_tmp = time_from;
        while (time_tmp.getTime() <= time_to.getTime()) {
            Date time_tmp_next = new Date(time_tmp.getTime() + interval);

            List<DoubleTimeObject> dataList_tmp = new ArrayList<DoubleTimeObject>();
            for (int i = index_src; i < dataList_servicedata_src.size(); i++) {
                index_src = i;
                DoubleTimeObject data = dataList_servicedata_src.get(i);
                if (data.data_time.getTime() >= time_tmp_next.getTime()) {
                    break;
                } else if (data.data_time.getTime() >= time_tmp.getTime()) {
                    dataList_tmp.add(data);
                }
            }
            DoubleTimeObject data_des = null;
            for (int i = index_des; i < dataList_servicedata_des.size(); i++) {
                index_des = i;
                DoubleTimeObject data = dataList_servicedata_des.get(i);
                if (data.data_time.getTime() > time_tmp.getTime()) {
                    break;
                } else if (data.data_time.getTime() == time_tmp.getTime()) {
                    data_des = data;
                    break;
                }
            }
            if (dataList_tmp.size() > 0) {
                double sum = 0.0;
                for (DoubleTimeObject data : dataList_tmp) {
                    sum += data.data_value;
                }
                if (data_des == null || Math.abs(data_des.data_value - sum) >= 0.001) {
                    if (error_from == null) {
                        error_from = time_tmp;
                    }
                    error_to = time_tmp;
                }
            }

            time_tmp = time_tmp_next;
        }
        if (error_from != null) {
            log.warn(building + "\t" + meter + "\t" + funcid + "\t" + time_period + "\t" + sdf.format(error_from) + "\t" + sdf.format(error_to));
        }
    }
}
